@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2017-2018 - Chris Griffith - MIT License
+# Copyright (c) 2017-2019 - Chris Griffith - MIT License
 """
 Improved dictionary access through dot notation with additional tools.
 """
@@ -29,6 +29,14 @@ except ImportError:
         yaml = None
         yaml_support = False
 
+wrapt_support = True
+
+try:
+    import wrapt
+except ImportError:
+    wrapt = None
+    wrapt_support = False
+
 if sys.version_info >= (3, 0):
     basestring = str
 else:
@@ -37,11 +45,12 @@ else:
 __all__ = ['Box', 'ConfigBox', 'BoxList', 'SBox',
            'BoxError', 'BoxKeyError']
 __author__ = 'Chris Griffith'
-__version__ = '3.2.2'
+__version__ = '3.4.4'
 
 BOX_PARAMETERS = ('default_box', 'default_box_attr', 'conversion_box',
                   'frozen_box', 'camel_killer_box', 'box_it_up',
-                  'box_safe_prefix', 'box_duplicates', 'ordered_box')
+                  'box_safe_prefix', 'box_duplicates', 'ordered_box',
+                  'box_intact_types')
 
 _first_cap_re = re.compile('(.)([A-Z][a-z]+)')
 _all_cap_re = re.compile('([a-z0-9])([A-Z])')
@@ -167,8 +176,8 @@ def _camel_killer(attr):
 
     s1 = _first_cap_re.sub(r'\1_\2', attr)
     s2 = _all_cap_re.sub(r'\1_\2', s1)
-    return re.sub('_+', '_', s2.casefold() if hasattr(s2, 'casefold') else
-                  s2.lower())
+    return re.sub('_+', '_', s2.casefold() if hasattr(s2, 'casefold')
+                  else s2.lower())
 
 
 def _recursive_tuples(iterable, box_class, recreate_tuples=False, **kwargs):
@@ -233,6 +242,7 @@ def _get_box_config(cls, kwargs):
         '__converted': set(),
         '__box_heritage': kwargs.pop('__box_heritage', None),
         '__created': False,
+        '__ordered_box_values': [],
         # Can be changed by user after box creation
         'default_box': kwargs.pop('default_box', False),
         'default_box_attr': kwargs.pop('default_box_attr', cls),
@@ -242,7 +252,8 @@ def _get_box_config(cls, kwargs):
         'camel_killer_box': kwargs.pop('camel_killer_box', False),
         'modify_tuples_box': kwargs.pop('modify_tuples_box', False),
         'box_duplicates': kwargs.pop('box_duplicates', 'ignore'),
-        'ordered_box': kwargs.pop('ordered_box', False)
+        'ordered_box': kwargs.pop('ordered_box', False),
+        'box_intact_types': tuple(kwargs.pop('box_intact_types', ())),
     }
 
 
@@ -262,6 +273,7 @@ class Box(dict):
     :param box_duplicates: "ignore", "error" or "warn" when duplicates exists
         in a conversion_box
     :param ordered_box: Preserve the order of keys entered into the box
+    :param box_intact_types: Keep data with given types intact
     """
 
     _protected_keys = dir({}) + ['to_dict', 'tree_view', 'to_json', 'to_yaml',
@@ -279,7 +291,7 @@ class Box(dict):
     def __init__(self, *args, **kwargs):
         self._box_config = _get_box_config(self.__class__, kwargs)
         if self._box_config['ordered_box']:
-            self._box_config['ordered_box_values'] = []
+            self._box_config['__ordered_box_values'] = []
         if (not self._box_config['conversion_box'] and
                 self._box_config['box_duplicates'] != "ignore"):
             raise BoxError('box_duplicates are only for conversion_boxes')
@@ -318,8 +330,8 @@ class Box(dict):
 
     def __add_ordered(self, key):
         if (self._box_config['ordered_box'] and
-                key not in self._box_config['ordered_box_values']):
-            self._box_config['ordered_box_values'].append(key)
+                key not in self._box_config['__ordered_box_values']):
+            self._box_config['__ordered_box_values'].append(key)
 
     def box_it_up(self):
         """
@@ -379,23 +391,24 @@ class Box(dict):
         return list(items)
 
     def get(self, key, default=None):
-        try:
-            return self[key]
-        except KeyError:
+        if key not in self:
+            if default is None and self._box_config['default_box']:
+                return self.__get_default(key)
             if isinstance(default, dict) and not isinstance(default, Box):
                 return Box(default)
             if isinstance(default, list) and not isinstance(default, BoxList):
                 return BoxList(default)
             return default
+        return self[key]
 
     def copy(self):
-        return self.__class__(super(self.__class__, self).copy())
+        return Box(super(Box, self).copy())
 
     def __copy__(self):
-        return self.__class__(super(self.__class__, self).copy())
+        return Box(super(Box, self).copy())
 
     def __deepcopy__(self, memodict=None):
-        out = self.__class__()
+        out = self.__class__(**self.__box_config())
         memodict = memodict or {}
         memodict[id(self)] = out
         for k, v in self.items():
@@ -421,7 +434,7 @@ class Box(dict):
 
     def keys(self):
         if self._box_config['ordered_box']:
-            return self._box_config['ordered_box_values']
+            return self._box_config['__ordered_box_values']
         return super(Box, self).keys()
 
     def values(self):
@@ -449,7 +462,9 @@ class Box(dict):
         return out
 
     def __convert_and_store(self, item, value):
-        if item in self._box_config['__converted']:
+        if (item in self._box_config['__converted'] or
+                (self._box_config['box_intact_types'] and
+                 isinstance(value, self._box_config['box_intact_types']))):
             return value
         if isinstance(value, dict) and not isinstance(value, Box):
             value = self.__class__(value, __box_heritage=(self, item),
@@ -492,6 +507,8 @@ class Box(dict):
             except KeyError:
                 value = object.__getattribute__(self, item)
         except AttributeError as err:
+            if item == "__getstate__":
+                raise AttributeError(item)
             if item == '_box_config':
                 raise BoxError('_box_config key must exist')
             kill_camel = self._box_config['camel_killer_box']
@@ -530,25 +547,20 @@ class Box(dict):
             raise AttributeError("Key name '{0}' is protected".format(key))
         if key == '_box_config':
             return object.__setattr__(self, key, value)
-        try:
-            object.__getattribute__(self, key)
-        except (AttributeError, UnicodeEncodeError):
-            if (key not in self.keys() and
-                    (self._box_config['conversion_box'] or
-                     self._box_config['camel_killer_box'])):
-                if self._box_config['conversion_box']:
-                    k = _conversion_checks(key, self.keys(),
-                                           self._box_config)
-                    self[key if not k else k] = value
-                elif self._box_config['camel_killer_box']:
-                    for each_key in self:
-                        if key == _camel_killer(each_key):
-                            self[each_key] = value
-                            break
-            else:
-                self[key] = value
+        if (key not in self.keys() and
+                (self._box_config['conversion_box'] or
+                 self._box_config['camel_killer_box'])):
+            if self._box_config['conversion_box']:
+                k = _conversion_checks(key, self.keys(),
+                                       self._box_config)
+                self[key if not k else k] = value
+            elif self._box_config['camel_killer_box']:
+                for each_key in self:
+                    if key == _camel_killer(each_key):
+                        self[each_key] = value
+                        break
         else:
-            object.__setattr__(self, key, value)
+            self[key] = value
         self.__add_ordered(key)
         self.__create_lineage()
 
@@ -557,8 +569,8 @@ class Box(dict):
             raise BoxError('Box is frozen')
         super(Box, self).__delitem__(key)
         if (self._box_config['ordered_box'] and
-                key in self._box_config['ordered_box_values']):
-            self._box_config['ordered_box_values'].remove(key)
+                key in self._box_config['__ordered_box_values']):
+            self._box_config['__ordered_box_values'].remove(key)
 
     def __delattr__(self, item):
         if self._box_config['frozen_box']:
@@ -567,15 +579,7 @@ class Box(dict):
             raise BoxError('"_box_config" is protected')
         if item in self._protected_keys:
             raise AttributeError("Key name '{0}' is protected".format(item))
-        try:
-            object.__getattribute__(self, item)
-        except AttributeError:
-            del self[item]
-        else:
-            object.__delattr__(self, item)
-        if (self._box_config['ordered_box'] and
-                item in self._box_config['ordered_box_values']):
-            self._box_config['ordered_box_values'].remove(item)
+        del self[item]
 
     def pop(self, key, *args):
         if args:
@@ -588,6 +592,8 @@ class Box(dict):
                 return args[0]
             else:
                 del self[key]
+                if isinstance(item, Box):
+                    item._box_config['__box_heritage'] = ()
                 return item
         try:
             item = self[key]
@@ -595,10 +601,12 @@ class Box(dict):
             raise BoxKeyError('{0}'.format(key))
         else:
             del self[key]
+            if isinstance(item, Box):
+                item._box_config['__box_heritage'] = ()
             return item
 
     def clear(self):
-        self._box_config['ordered_box_values'] = []
+        self._box_config['__ordered_box_values'] = []
         super(Box, self).clear()
 
     def popitem(self):
@@ -639,33 +647,26 @@ class Box(dict):
                 out_dict[k] = v.to_list()
         return out_dict
 
-    def update(self, item=None, **kwargs):
-        if not item:
-            item = kwargs
-        iter_over = item.items() if hasattr(item, 'items') else item
-        for k, v in iter_over:
-            if isinstance(v, dict):
-                # Box objects must be created in case they are already
-                # in the `converted` box_config set
-                v = self.__class__(v)
-                if k in self and isinstance(self[k], dict):
-                    self[k].update(v)
-                    continue
-            if isinstance(v, list):
-                v = BoxList(v)
-            try:
-                self.__setattr__(k, v)
-            except (AttributeError, TypeError):
-                self.__setitem__(k, v)
+    def update(self, E=None, **F):
+        if E:
+            if hasattr(E, 'keys'):
+                for k in E:
+                    self[k] = self.__convert_and_store(k, E[k])
+            else:
+                for k, v in E:
+                    self[k] = self.__convert_and_store(k, v)
+        for k in F:
+            self[k] = self.__convert_and_store(k, F[k])
 
     def setdefault(self, item, default=None):
         if item in self:
             return self[item]
 
         if isinstance(default, dict):
-            default = self.__class__(default)
+            default = self.__class__(default, **self.__box_config())
         if isinstance(default, list):
-            default = BoxList(default)
+            default = BoxList(default,
+                              box_class=self.__class__, **self.__box_config())
         self[item] = default
         return default
 
@@ -766,7 +767,7 @@ class BoxList(list):
     def __init__(self, iterable=None, box_class=Box, **box_options):
         self.box_class = box_class
         self.box_options = box_options
-        self.box_org_ref = id(iterable)
+        self.box_org_ref = self.box_org_ref = id(iterable) if iterable else 0
         if iterable:
             for x in iterable:
                 self.append(x)
@@ -788,14 +789,24 @@ class BoxList(list):
             raise BoxError('BoxList is frozen')
         super(BoxList, self).__setitem__(key, value)
 
+    def _is_intact_type(self, obj):
+        try:
+            if (self.box_options.get('box_intact_types') and
+                    isinstance(obj, self.box_options['box_intact_types'])):
+                return True
+        except AttributeError as err:
+            if 'box_options' in self.__dict__:
+                raise err
+        return False
+
     def append(self, p_object):
-        if isinstance(p_object, dict):
+        if isinstance(p_object, dict) and not self._is_intact_type(p_object):
             try:
                 p_object = self.box_class(p_object, **self.box_options)
             except AttributeError as err:
                 if 'box_class' in self.__dict__:
                     raise err
-        elif isinstance(p_object, list):
+        elif isinstance(p_object, list) and not self._is_intact_type(p_object):
             try:
                 p_object = (self if id(p_object) == self.box_org_ref else
                             BoxList(p_object))
@@ -809,9 +820,9 @@ class BoxList(list):
             self.append(item)
 
     def insert(self, index, p_object):
-        if isinstance(p_object, dict):
+        if isinstance(p_object, dict) and not self._is_intact_type(p_object):
             p_object = self.box_class(p_object, **self.box_options)
-        elif isinstance(p_object, list):
+        elif isinstance(p_object, list) and not self._is_intact_type(p_object):
             p_object = (self if id(p_object) == self.box_org_ref else
                         BoxList(p_object))
         super(BoxList, self).insert(index, p_object)
@@ -828,7 +839,7 @@ class BoxList(list):
                        **self.box_options)
 
     def __deepcopy__(self, memodict=None):
-        out = self.__class__()
+        out = self.__class__(**self.box_options)
         memodict = memodict or {}
         memodict[id(self)] = out
         for k in self:
@@ -1080,6 +1091,12 @@ class ConfigBox(Box):
     def __repr__(self):
         return '<ConfigBox: {0}>'.format(str(self.to_dict()))
 
+    def copy(self):
+        return ConfigBox(super(ConfigBox, self).copy())
+
+    def __copy__(self):
+        return ConfigBox(super(ConfigBox, self).copy())
+
 
 class SBox(Box):
     """
@@ -1105,3 +1122,86 @@ class SBox(Box):
 
     def __repr__(self):
         return '<ShorthandBox: {0}>'.format(str(self.to_dict()))
+
+    def copy(self):
+        return SBox(super(SBox, self).copy())
+
+    def __copy__(self):
+        return SBox(super(SBox, self).copy())
+
+
+if wrapt_support:
+
+    class BoxObject(wrapt.ObjectProxy):
+        """
+        Wrapper for any Python object with a Box as __dict__.
+
+        Simple Usage:
+
+        import requests
+        url = 'https://raw.githubusercontent.com/cdgriffith/Box/master/box.py'
+        session = BoxObject(requests.Session())
+        session.source_code = session.get(url).text
+
+        :param wrapped: Wrapped Object.
+        :param box_class: Custom internal Box class
+        :param args: Arguments to fill Box
+        :param kwargs: Keyword arguments to fill Box
+        """
+
+        def __init__(self, wrapped=None, *args, **kwargs):
+            """Initialize Box Object with __dict__ as a Box."""
+            super(BoxObject, self).__init__(wrapped)
+            box_class = kwargs.pop('box_class', Box)
+            try:
+                base_dict = super(BoxObject, self).__getattr__('__dict__')
+                if args:
+                    raise TypeError('Cannot pass dictionary arguments when '
+                                    'internal object has __dict__ attributes. '
+                                    'Pass arguments by keyword instead.')
+                box = box_class(base_dict, **kwargs)
+            except AttributeError:
+                box = box_class(*args, **kwargs)
+            super(BoxObject, self).__setattr__('__dict__', box)
+
+        def __call__(self, *args, **kwargs):
+            """Call Method for Callable Objects."""
+            return self.__wrapped__(*args, **kwargs)
+
+        def __getattr__(self, name):
+            """Get Attribute from Wrapped Object or from Box."""
+            try:
+                return super(BoxObject, self).__getattr__(name)
+            except AttributeError as error:
+                try:
+                    return self.__dict__[name]
+                except KeyError:
+                    raise error
+
+        def __setattr__(self, name, value):
+            """Set Attribute in Wrapped Object or Box."""
+            if name == '__dict__':
+                raise TypeError('cannot set __dict__')
+            elif hasattr(self.__wrapped__, name):
+                setattr(self.__wrapped__, name, value)
+            else:
+                self.__dict__[name] = value
+
+        def __delattr__(self, name):
+            """Delete Attribute in Wrapped Object or Box."""
+            if name == '__dict__':
+                super(BoxObject, self).__setattr__(
+                    '__dict__',
+                    getattr(self.__wrapped__, '__dict__', {})
+                )
+            else:
+                try:
+                    delattr(self.__wrapped__, name)
+                except AttributeError as error:
+                    try:
+                        del self.__dict__[name]
+                    except KeyError:
+                        raise error
+
+
+    __all__ += ['BoxObject']
